@@ -13,20 +13,11 @@ my $PORT = shift @ARGV or die "Need PORT";
 
 my $loop = IO::Async::Loop->new;
 
-my $socket;
-
-$loop->connect(
+my $socket = $loop->connect(
    host     => $HOST,
    service  => $PORT,
    socktype => 'stream',
-
-   on_connected => sub { $socket = shift },
-
-   on_resolve_error => sub { die "Cannot resolve - $_[0]\n" },
-   on_connect_error => sub { die "Cannot connect\n" },
-);
-
-$loop->loop_once until defined $socket;
+)->get;
 
 # $socket is just an IO::Socket reference
 my $peeraddr = $socket->peerhost . ":" . $socket->peerport;
@@ -37,12 +28,6 @@ print STDERR "Connected to $peeraddr\n";
 # easily without a temporary variable
 my ( $socketstream, $stdiostream );
 
-my $quit_mergepoint = IO::Async::MergePoint->new(
-   needs => [qw[ socket stdio ]],
-
-   on_finished => sub { $loop->loop_stop },
-);
-
 $socketstream = IO::Async::Stream->new(
    handle => $socket,
 
@@ -58,7 +43,6 @@ $socketstream = IO::Async::Stream->new(
 
    on_closed => sub {
       print STDERR "Closed connection to $peeraddr\n";
-      $quit_mergepoint->done( 'socket' );
       $stdiostream->close_when_empty;
    },
 );
@@ -76,10 +60,9 @@ $stdiostream = IO::Async::Stream->new_for_stdio(
    },
 
    on_closed => sub {
-      $quit_mergepoint->done( 'stdio' );
       $socketstream->close_when_empty;
    },
 );
 $loop->add( $stdiostream );
 
-$loop->loop_forever;
+$loop->await_all( $socketstream->new_close_future, $stdiostream->new_close_future );
