@@ -48,9 +48,11 @@ $loop->add( $listener );
 $listener->listen(
    service  => $PORT,
    socktype => 'stream',
+)->on_done( sub {
+   my ( $listener ) = @_;
+   my $socket = $listener->read_handle;
 
-   on_resolve_error => sub { die "Cannot resolve - $_[0]\n"; },
-   on_listen_error  => sub { die "Cannot listen\n"; },
-);
+   printf STDERR "Listening on %s:%d\n", $socket->sockhost, $socket->sockport;
+})->get;
 
-$loop->loop_forever;
+$loop->run;
